@@ -6,18 +6,16 @@ use base qw( Class::Accessor::Fast );
 use Data::Dumper;
 
 BEGIN {
-    __PACKAGE__->mk_accessors(qw/_twitter callback_url consumer_key consumer_secret/);
+    __PACKAGE__->mk_accessors(qw/
+        _twitter twitter_user callback_url consumer_key consumer_secret
+    /);
 }
 
-our $VERSION = "0.01";
+our $VERSION = '0.02000';
 
 use Catalyst::Exception ();
 use Net::Twitter;
 
-use Catalyst qw/
-	Session::PerUser
-/;
-
 sub new {
     my ($class, $config, $c, $realm) = @_;
     my $self = {};
@@ -45,8 +43,8 @@ sub new {
     return $self;
 }
 
-sub authenticate {
-    my ( $self, $c, $realm, $authinfo ) = @_;
+sub authenticate_twitter {
+    my ( $self, $c ) = @_;
 
 	if (!$c->user_session->{'request_token'} || !$c->user_session->{'request_token_secret'} || !$c->req->params->{'oauth_verifier'}) {
 		die 'no request token present, or no verifier';
@@ -55,6 +53,8 @@ sub authenticate {
 	my $token = $c->user_session->{'request_token'};
 	my $token_secret = $c->user_session->{'request_token_secret'};
 	my $verifier = $c->req->params->{'oauth_verifier'};
+	my $access_token = $c->user_session->{'access_token'};
+	my $access_token_secret = $c->user_session->{'access_token_secret'};
 
     # Create a Net::Twitter instance
     $self->_twitter(Net::Twitter->new({ 
@@ -63,18 +63,22 @@ sub authenticate {
         'consumer_secret'	=> $self->consumer_secret,
 	}));
 
-	$self->_twitter->request_token($token);
-    $self->_twitter->request_token_secret($token_secret);
+	if (!$access_token && !$access_token_secret) {
+		$self->_twitter->request_token($token);
+    	$self->_twitter->request_token_secret($token_secret);
 
-	my($access_token, $access_token_secret) = $self->_twitter->request_access_token('verifier' => $verifier);
+		($access_token, $access_token_secret) = $self->_twitter->request_access_token('verifier' => $verifier);
+		# this is in case we need to register the user after the oauth process
+		$c->user_session->{'access_token'} = $access_token;
+		$c->user_session->{'access_token_secret'} = $access_token_secret;
+	}
 
 	# get the user
 	$self->_twitter->access_token($access_token);
     $self->_twitter->access_token_secret($access_token_secret);
 
-	my $twitter_user_hash;
-	eval {
-		$twitter_user_hash = $self->_twitter->verify_credentials();
+	my $twitter_user_hash = eval {
+		$self->_twitter->verify_credentials;
 	};
 
 	if ($@ || !$twitter_user_hash) {
@@ -85,9 +89,19 @@ sub authenticate {
 	$twitter_user_hash->{'access_token'} = $access_token;
 	$twitter_user_hash->{'access_token_secret'} = $access_token_secret;
 
-	if (!$authinfo) {
+    $self->twitter_user( $twitter_user_hash );
+
+    return $twitter_user_hash;
+}
+
+sub authenticate {
+    my ( $self, $c, $realm, $authinfo ) = @_;
+
+	unless ($authinfo) {
+        $self->authenticate_twitter( $c ) unless $self->twitter_user;
+
 		$authinfo = {
-			'twitter_user_id'	=> $twitter_user_hash->{'id'},
+			'twitter_user_id'	=> $self->twitter_user->{'id'},
 		};
 	}
 
@@ -95,10 +109,11 @@ sub authenticate {
 
 	if (ref $user_obj) {
 		if ($user_obj->result_source->has_column('twitter_user') && $user_obj->result_source->has_column('twitter_access_token') && $user_obj->result_source->has_column('twitter_access_token_secret')) {
+            my $twitter_user = $self->twitter_user;
 			$user_obj->update({
-				'twitter_user'					=> $twitter_user_hash->{'screen_name'},
-				'twitter_access_token'			=> $access_token,
-				'twitter_access_token_secret'	=> $access_token_secret,
+				'twitter_user'					=> $twitter_user->{'screen_name'},
+				'twitter_access_token'			=> $twitter_user->{access_token},
+				'twitter_access_token_secret'	=> $twitter_user->{access_token_secret},
 			});
 		}
 		return $user_obj;
@@ -120,6 +135,8 @@ sub authenticate_twitter_url {
     my $uri = $self->_twitter->get_authorization_url( 'callback'	=> $c->config->{'twitter_callback_url'} || $self->callback_url );
 	$c->user_session->{'request_token'} = $self->_twitter->request_token;
 	$c->user_session->{'request_token_secret'} = $self->_twitter->request_token_secret;
+	$c->user_session->{'access_token'} = '';
+	$c->user_session->{'access_token_secret'} = '';
 
     return $uri;
 }
@@ -137,6 +154,7 @@ In MyApp.pm
     Session
     Session::Store::FastMmap
     Session::State::Cookie
+	Session::PerUser
  /;
  
  MyApp->config(
@@ -233,6 +251,34 @@ in your main user table. If the other keys are present they will be
 updated on login with Twitter's most up-to-date information for that
 user.
 
+=head2 authenticate_twitter( )
+
+Only performs the twitter authentication. Returns a hashref containing
+the user's information given by Twitter (see C<authenticate()> above for
+the lists of keys returned).
+
+=head2 twitter_user()
+
+Contains the user's twitter information after a successful twitter
+authentication via C<authenticate_twitter()> or
+C<authenticate()>. Useful if, for example, you want to create users
+on-the-fly:
+
+    sub twitter_callback :Path( 'twitter/callback' ) {
+        my ($self, $c) = @_;
+
+        my $twitter = $c->get_auth_realm('twitter')->credential;
+        my $user =  $twitter->authenticate( $c );
+
+        # properly authenticated against twitter,
+        # user just doesn't exist yet
+        if ( !$user and  $twitter->twitter_user ) {
+            $user = $self->model->create_user( $twitter->twitter_user );
+        }
+
+        # etc
+    }
+
 =head1 AUTHOR
 
 Jesse Stay
